# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: gcsystemmsgs.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='gcsystemmsgs.proto',
  package='dota',
  syntax='proto2',
  serialized_options=_b('H\001\220\001\000'),
  serialized_pb=_b('\n\x12gcsystemmsgs.proto\x12\x04\x64ota*\xf0\x01\n\x06\x45SOMsg\x12\x13\n\x0fk_ESOMsg_Create\x10\x15\x12\x13\n\x0fk_ESOMsg_Update\x10\x16\x12\x14\n\x10k_ESOMsg_Destroy\x10\x17\x12\x1c\n\x18k_ESOMsg_CacheSubscribed\x10\x18\x12\x1e\n\x1ak_ESOMsg_CacheUnsubscribed\x10\x19\x12\x1b\n\x17k_ESOMsg_UpdateMultiple\x10\x1a\x12%\n!k_ESOMsg_CacheSubscriptionRefresh\x10\x1c\x12$\n k_ESOMsg_CacheSubscribedUpToDate\x10\x1d*\xc2\x03\n\x10\x45GCBaseClientMsg\x12\x18\n\x13k_EMsgGCPingRequest\x10\xb9\x17\x12\x19\n\x14k_EMsgGCPingResponse\x10\xba\x17\x12&\n!k_EMsgGCToClientPollConvarRequest\x10\xbb\x17\x12\'\n\"k_EMsgGCToClientPollConvarResponse\x10\xbc\x17\x12\"\n\x1dk_EMsgGCCompressedMsgToClient\x10\xbd\x17\x12)\n$k_EMsgGCCompressedMsgToClient_Legacy\x10\x8b\x04\x12#\n\x1ek_EMsgGCToClientRequestDropped\x10\xbe\x17\x12\x1a\n\x15k_EMsgGCClientWelcome\x10\xa4\x1f\x12\x1a\n\x15k_EMsgGCServerWelcome\x10\xa5\x1f\x12\x18\n\x13k_EMsgGCClientHello\x10\xa6\x1f\x12\x18\n\x13k_EMsgGCServerHello\x10\xa7\x1f\x12#\n\x1ek_EMsgGCClientConnectionStatus\x10\xa9\x1f\x12#\n\x1ek_EMsgGCServerConnectionStatus\x10\xaa\x1f\x42\x05H\x01\x90\x01\x00')
)

_ESOMSG = _descriptor.EnumDescriptor(
  name='ESOMsg',
  full_name='dota.ESOMsg',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='k_ESOMsg_Create', index=0, number=21,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_ESOMsg_Update', index=1, number=22,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_ESOMsg_Destroy', index=2, number=23,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_ESOMsg_CacheSubscribed', index=3, number=24,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_ESOMsg_CacheUnsubscribed', index=4, number=25,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_ESOMsg_UpdateMultiple', index=5, number=26,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_ESOMsg_CacheSubscriptionRefresh', index=6, number=28,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_ESOMsg_CacheSubscribedUpToDate', index=7, number=29,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=29,
  serialized_end=269,
)
_sym_db.RegisterEnumDescriptor(_ESOMSG)

ESOMsg = enum_type_wrapper.EnumTypeWrapper(_ESOMSG)
_EGCBASECLIENTMSG = _descriptor.EnumDescriptor(
  name='EGCBaseClientMsg',
  full_name='dota.EGCBaseClientMsg',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='k_EMsgGCPingRequest', index=0, number=3001,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EMsgGCPingResponse', index=1, number=3002,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EMsgGCToClientPollConvarRequest', index=2, number=3003,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EMsgGCToClientPollConvarResponse', index=3, number=3004,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EMsgGCCompressedMsgToClient', index=4, number=3005,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EMsgGCCompressedMsgToClient_Legacy', index=5, number=523,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EMsgGCToClientRequestDropped', index=6, number=3006,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EMsgGCClientWelcome', index=7, number=4004,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EMsgGCServerWelcome', index=8, number=4005,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EMsgGCClientHello', index=9, number=4006,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EMsgGCServerHello', index=10, number=4007,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EMsgGCClientConnectionStatus', index=11, number=4009,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='k_EMsgGCServerConnectionStatus', index=12, number=4010,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=272,
  serialized_end=722,
)
_sym_db.RegisterEnumDescriptor(_EGCBASECLIENTMSG)

EGCBaseClientMsg = enum_type_wrapper.EnumTypeWrapper(_EGCBASECLIENTMSG)
k_ESOMsg_Create = 21
k_ESOMsg_Update = 22
k_ESOMsg_Destroy = 23
k_ESOMsg_CacheSubscribed = 24
k_ESOMsg_CacheUnsubscribed = 25
k_ESOMsg_UpdateMultiple = 26
k_ESOMsg_CacheSubscriptionRefresh = 28
k_ESOMsg_CacheSubscribedUpToDate = 29
k_EMsgGCPingRequest = 3001
k_EMsgGCPingResponse = 3002
k_EMsgGCToClientPollConvarRequest = 3003
k_EMsgGCToClientPollConvarResponse = 3004
k_EMsgGCCompressedMsgToClient = 3005
k_EMsgGCCompressedMsgToClient_Legacy = 523
k_EMsgGCToClientRequestDropped = 3006
k_EMsgGCClientWelcome = 4004
k_EMsgGCServerWelcome = 4005
k_EMsgGCClientHello = 4006
k_EMsgGCServerHello = 4007
k_EMsgGCClientConnectionStatus = 4009
k_EMsgGCServerConnectionStatus = 4010


DESCRIPTOR.enum_types_by_name['ESOMsg'] = _ESOMSG
DESCRIPTOR.enum_types_by_name['EGCBaseClientMsg'] = _EGCBASECLIENTMSG
_sym_db.RegisterFileDescriptor(DESCRIPTOR)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



import logging
import six
from eventemitter import EventEmitter
from dota2.enums import EDOTAGCMsg, DOTAChatChannelType_t
from dota2.protobufs.dota_gcmessages_client_chat_pb2 import CMsgDOTAJoinChatChannelResponse,\
                                                            CMsgDOTAChatChannelFullUpdate,\
                                                            CMsgDOTAOtherJoinedChatChannel,\
                                                            CMsgDOTAOtherLeftChatChannel,\
                                                            CMsgDOTAChatChannelMemberUpdate


class ChatBase(object):
    def __init__(self):
        super(ChatBase, self).__init__()
        name = "%s.channels" % self.__class__.__name__
        self.channels = ChannelManager(self, name)


class ChannelManager(EventEmitter):
    EVENT_JOINED_CHANNEL = 'channel_joined'
    """When the client join a channel.

    :param channel: channel instance
    :type  channel: :class:`ChatChannel`
    """
    EVENT_LEFT_CHANNEL = 'channel_left'
    """When the client leaves a channel.

    :param channel: channel instance
    :type  channel: :class:`ChatChannel`
    """
    EVENT_MESSAGE = 'message'
    """On a new channel message

    :param channel: channel instance
    :type  channel: :class:`ChatChannel`
    :param message: message data
    :type  message: `CMsgDOTAChatMessage <https://github.com/ValvePython/dota2/blob/6cb1008f3070e008e9bed9521fad8d1438123aa1/protobufs/dota_gcmessages_client_chat.proto#L86-L122>`_
    """
    EVENT_CHANNEL_MEMBERS_UPDATE = 'members_update'
    """When users join/leave a channel

    :param channel: channel instance
    :type  channel: :class:`ChatChannel`
    :param joined: list of members who joined
    :type  joined: list
    :param left: list of members who left
    :type  left: list
    """

    def emit(self, event, *args):
        if event is not None:
            self._LOG.debug("Emit event: %s" % repr(event))
        EventEmitter.emit(self, event, *args)

    def __init__(self, dota_client, logger_name):
        super(ChannelManager, self).__init__()

        self._LOG = logging.getLogger(logger_name if logger_name else self.__class__.__name__)
        self._dota = dota_client
        self._channels = {}
        self._channels_by_name = {}

        # register our handlers
        self._dota.on('notready', self._cleanup)
        self._dota.on(EDOTAGCMsg.EMsgGCJoinChatChannelResponse, self._handle_join_response)
        self._dota.on(EDOTAGCMsg.EMsgGCChatMessage, self._handle_message)
        self._dota.on(EDOTAGCMsg.EMsgGCOtherJoinedChannel, self._handle_members_update)
        self._dota.on(EDOTAGCMsg.EMsgGCOtherLeftChannel, self._handle_members_update)
        self._dota.on(EDOTAGCMsg.EMsgDOTAChatChannelMemberUpdate, self._handle_members_update)

    def __repr__(self):
        return "<ChannelManager(): %d channels>" % (
                    len(self),
                    )

    def _cleanup(self):
        self._channels.clear()
        self._channels_by_name.clear()

    def _remove_channel(self, channel_id):
        channel = self._channels.pop(channel_id, None)
        self._channels_by_name.pop((channel.name, channel.type), None)

    def __contains__(self, key):
        return (key in self._channels) or (key in self._channels_by_name)

    def __getitem__(self, key):
        if isinstance(key, tuple):
            return self._channels_by_name[key]
        else:
            return self._channels[key]

    def __len__(self):
        return len(self._channels)

    def __iter__(self):
        return six.itervalues(self._channels)

    def _handle_join_response(self, message):
        key = (message.channel_name, message.channel_type)
        self.emit(('join_result',) + key, message.result)

        if message.result == message.JOIN_SUCCESS:
            if message.channel_id in self:
                channel = self[message.channel_id]
            else:
                channel = ChatChannel(self, message)

                self._channels[channel.id] = channel
                self._channels_by_name[key] = channel

            self.emit(self.EVENT_JOINED_CHANNEL, channel)

    def _handle_message(self, message):
        if message.channel_id in self:
            self.emit(self.EVENT_MESSAGE, self[message.channel_id], message)

    def _handle_members_update(self, message):
        if message.channel_id in self:
            channel = self[message.channel_id]
            joined = []
            left = []

            if isinstance(message, CMsgDOTAOtherLeftChatChannel):
                left.append(message.steam_id or message.channel_user_id)
            elif isinstance(message, CMsgDOTAOtherJoinedChatChannel):
                joined.append(message.steam_id or message.channel_user_id)
            elif isinstance(message, CMsgDOTAChatChannelMemberUpdate):
                left = list(message.left_steam_ids)
                joined = list(map(lambda x: x.steam_id or x.channel_user_id, message.joined_members))
            elif isinstance(message, CMsgDOTAChatChannelFullUpdate):
                pass

            channel._process_members_from_proto(message)

            if joined or left:
                self.emit(self.EVENT_CHANNEL_MEMBERS_UPDATE, channel, joined, left)

    def join_channel(self, channel_name, channel_type=DOTAChatChannelType_t.DOTAChannelType_Custom):
        """Join a chat channel

        :param channel_name: channel name
        :type  channel_name: str
        :param channel_type: channel type
        :type  channel_type: :class:`.DOTAChatChannelType_t`
        :return: join result
        :rtype: int

        Response event: :attr:`EVENT_JOINED_CHANNEL`
        """
        if self._dota.verbose_debug:
            self._LOG.debug("Request to join chat channel: %s", channel_name)

        self._dota.send(EDOTAGCMsg.EMsgGCJoinChatChannel, {
            "channel_name": channel_name,
            "channel_type": channel_type
        })

        resp = self.wait_event(('join_result', channel_name, channel_type), timeout=25)

        if resp:
            return resp[0]
        else:
            return None

    def join_lobby_channel(self):
        """
        Join the lobby channel if the client is in a lobby.

        Response event: :attr:`EVENT_JOINED_CHANNEL`
        """
        if self._dota.lobby:
            key = "Lobby_%s" % self._dota.lobby.lobby_id, DOTAChatChannelType_t.DOTAChannelType_Lobby
            return self.join_channel(*key)

    @property
    def lobby(self):
        """References lobby channel if client has joined it

        :return: channel instance
        :rtype: :class:`.ChatChannel`
        """
        if self._dota.lobby:
            key = "Lobby_%s" % self._dota.lobby.lobby_id, DOTAChatChannelType_t.DOTAChannelType_Lobby
            return self._channels_by_name.get(key, None)

    def join_party_channel(self):
        """
        Join the lobby channel if the client is in a lobby.

        Response event: :attr:`EVENT_JOINED_CHANNEL`
        """
        if self._dota.party:
            key = "Party_%s" % self._dota.party.party_id, DOTAChatChannelType_t.DOTAChannelType_Party
            return self.join_channel(*key)

    @property
    def party(self):
        """References party channel if client has joined it

        :return: channel instance
        :rtype: :class:`.ChatChannel`
        """
        if self._dota.party:
            key = "Party_%s" % self._dota.party.party_id, DOTAChatChannelType_t.DOTAChannelType_Party
            return self._channels_by_name.get(key, None)

    def get_channel_list(self):
        """
        Requests a list of chat channels from the GC.

        :return: List of chat channels
        :rtype: `CMsgDOTAChatGetUserListResponse <https://github.com/ValvePython/dota2/blob/6cb1008f3070e008e9bed9521fad8d1438123aa1/protobufs/dota_gcmessages_client_chat.proto#L210-L220>`_, ``None``
        """
        if self._dota.verbose_debug:
            self._LOG.debug("Requesting channel list from GC.")

        jobid = self._dota.send_job(EDOTAGCMsg.EMsgGCRequestChatChannelList, {})
        resp = self._dota.wait_msg(jobid, timeout=25)

        return resp

    def leave_channel(self, channel_id):
        if channel_id in self:
            channel = self[channel_id]

            if self._dota.verbose_debug:
                self._LOG.debug("Leaving chat channel: %s", repr(channel))

            self._dota.send(EDOTAGCMsg.EMsgGCLeaveChatChannel, {
                "channel_id": channel_id
            })

            self._remove_channel(channel_id)
            self.emit(self.EVENT_LEFT_CHANNEL, channel)


class ChatChannel(object):
    def __init__(self, channel_manager, join_data):
        self._manager = channel_manager
        self._dota = self._manager._dota
        self._LOG = self._manager._LOG

        self.members = {}
        self.id = join_data.channel_id
        self.name = join_data.channel_name
        self.type = join_data.channel_type
        self.user_id = join_data.channel_user_id
        self.max_members = join_data.max_members

        self._process_members_from_proto(join_data)

    def __repr__(self):
        return "<ChatChannel(%s, %s, %s)>" % (
                    self.id,
                    repr(self.name),
                    self.type,
                    )

    def _process_members_from_proto(self, data):
        if isinstance(data, CMsgDOTAOtherLeftChatChannel):
            self.members.pop(data.steam_id or data.channel_user_id, None)
            return
        elif isinstance(data, CMsgDOTAOtherJoinedChatChannel):
            members = [data]
        elif isinstance(data, CMsgDOTAJoinChatChannelResponse):
            members = data.members
        elif isinstance(data, CMsgDOTAChatChannelMemberUpdate):
            for steam_id in data.left_steam_ids:
                self.members.pop(steam_id, None)

            members = data.joined_members

        for member in members:
            self.members[member.steam_id or member.channel_user_id] = (
                member.persona_name,
                member.status
            )


    def leave(self):
        """Leave channel"""
        self._manager.leave_channel(self.id)

    def send(self, message):
        """Send a message to the channel

        :param message: message text
        :type  message: str
        """
        self._dota.send(EDOTAGCMsg.EMsgGCChatMessage, {
            "channel_id": self.id,
            "text": message
        })

    def share_lobby(self):
        """Share current lobby to the channel"""
        if self._dota.lobby:
            self._dota.send(EDOTAGCMsg.EMsgGCChatMessage, {
                "channel_id": self.id,
                "share_lobby_id": self._dota.lobby.lobby_id,
                "share_lobby_passkey": self._dota.lobby.pass_key
            })

    def flip_coin(self):
        """Flip a coin"""
        self._dota.send(EDOTAGCMsg.EMsgGCChatMessage, {
            "channel_id": self.id,
            "coin_flip": True
        })

    def roll_dice(self, rollmin=1, rollmax=100):
        """Roll a dice

        :param rollmin: dice min value
        :type  rollmin: int
        :param rollmax: dice max value
        :type  rollmax: int
        """
        self._dota.send(EDOTAGCMsg.EMsgGCChatMessage, {
            "channel_id": self.id,
            "dice_roll": {
                "roll_min": dmin,
                "roll_max": dmax
            }
        })




def replay_url(match_id, cluster, replay_salt, app_id=570):
    """Form url for match replay

    :param match_id: match id
    :type  match_id: :class:`int`
    :param cluster: cluster the match is saved on
    :type  cluster: :class:`int`
    :param replay_salt: salt linked to the replay
    :type  replay_salt: :class:`int`
    :param app_id: (optional) app_id for dota
    :type  app_id: :class:`int`
    :return: url to download the replay of a specific match
    :rtype: :class:`str`
    """
    return 'http://replay{0}.valve.net/{1}/{2}_{3}.dem.bz2'.format(cluster, app_id, match_id, replay_salt)


def replay_url_from_match(match, app_id=570):
    """Form url for match replay

    :param match: `CMsgDOTAMatch <https://github.com/ValvePython/dota2/blob/e06c81c03579a912fcca829766ee590075ae97dc/protobufs/dota_gcmessages_common.proto#L866-L1001>`_
    :type  match: proto message
    :param app_id: (optional) app_id for dota
    :type  app_id: :class:`int`
    :return: url to download the replay of a specific match, None if match has not all the information
    :rtype: :class:`str`, :class:`None`
    """
    if match.HasField('match_id') and match.HasField('cluster') and match.HasField('replay_salt'):
        return replay_url(match.match_id, match.cluster, match.replay_salt, app_id)


def metadata_url(match_id, cluster, replay_salt, app_id=570):
    """Form url for match metadata file

    :param match_id: match id
    :type  match_id: :class:`int`
    :param cluster: cluster the match is saved on
    :type  cluster: :class:`int`
    :param replay_salt: salt linked to the replay
    :type  replay_salt: :class:`int`
    :param app_id: (optional) app_id for dota
    :type  app_id: :class:`int`
    :return: url to download the metadata of a specific match
    :rtype: :class:`str`
    """
    return 'http://replay{0}.valve.net/{1}/{2}_{3}.meta.bz2'.format(cluster, app_id, match_id, replay_salt)


def metadata_url_from_match(match, app_id=570):
    """Form url for match metadata file

    :param match: `CMsgDOTAMatch <https://github.com/ValvePython/dota2/blob/e06c81c03579a912fcca829766ee590075ae97dc/protobufs/dota_gcmessages_common.proto#L866-L1001>`_
    :type  match: proto message
    :param app_id: (optional) app_id for dota
    :type  app_id: :class:`int`
    :return: url to download the metadata of a specific match, None if match has not all the information
    :rtype: :class:`str`, :class:`None`
    """
    if match.HasField('match_id') and match.HasField('cluster') and match.HasField('replay_salt'):
        return metadata_url(match.match_id, match.cluster, match.replay_salt, app_id)
